#!/bin/bash
#
#
#                           Ezanvakti Temel İşlevler Bileşeni 1.5
#

declare -r AD="ezanvakti"
SURUM="7.1"
DUZELTME="20200428"

declare -r BILESEN_DIZINI="/usr/lib/${AD}"
declare -r VERI_DIZINI="/usr/share/${AD}"

EZANVAKTI_DIZINI="${XDG_CONFIG_HOME:-$HOME/.config}/${AD}"
EZANVAKTI_ONT_AYAR=/etc/${AD}.conf
EZANVAKTI_AYAR="${EZANVAKTI_DIZINI}/ayarlar"


[[ ! -d ${EZANVAKTI_DIZINI} ]] && mkdir -p "${EZANVAKTI_DIZINI}"
[[ ! -f ${EZANVAKTI_AYAR}   ]] && sed "s:\(EZANVERI_DIZINI=\).*:\1\'${EZANVAKTI_DIZINI}\':" \
  < "${EZANVAKTI_ONT_AYAR}" > "${EZANVAKTI_AYAR}"

# Öntanımlı ayarları al.
. "${EZANVAKTI_ONT_AYAR}"

#  Öntanımlı ayarlardan gelen değerlerin üzerine yazılacak.
#
# Kullanıcı ayarlarını al.
. "${EZANVAKTI_AYAR}" || {
  printf '%s: %s dosyası okunabilir değil.\n' \
    "${AD}" "${EZANVAKTI_AYAR}"
  exit 1
}

EZANVERI="${EZANVAKTI_DIZINI}/${EZANVERI_ADI}"
MPLAYER="mplayer -really-quiet -softvol -volume $SES"
declare -A YUKLENEN_BILESENLER

acilisa_baslatici_ekle() {
  local baslatici_dizini baslatici

  baslatici_dizini="${XDG_CONFIG_HOME:-$HOME/.config}/autostart/"
  baslatici="${baslatici_dizini}/${AD}.desktop"

  [[ ! -d ${baslatici_dizini} ]] && mkdir -p "${baslatici_dizini}"
  if (( ACILISTA_BASLAT ))
  then
      if [[ ! -f ${baslatici} ]]
      then
          printf '%s\n%s\n%s\n%s\n%s\n%s'\
            '[Desktop Entry]'\
            'Type=Application'\
            'Name=ezanvakti'\
            'Exec=/usr/lib/ezanvakti/ezanvakti-sleep 657a616e76616b7469'\
            'Comment=GNU/Linux icin ezan vakti bildirici'\
            'Terminal=false' > "${baslatici}"
          chmod 755 "${baslatici}"
      fi
  else
      [[ -f ${baslatici} ]] && rm -f "${baslatici}"
  fi
}

# Karar verme önceliği env olarak tanımlıysa RENK değişkenine verildi.
# RENK null ise ayarlar dosyasından okunan RENK_KULLAN değerini kullanacak.
#
# Renk kullan sıfırsa renkleri sıfırla.
renk_denetle() {
  [[ ! -t 1 ]] || [[ ! $DISPLAY ]] && { RENK_KULLAN=0; RENK=0; }
    (( ! ${RENK:-RENK_KULLAN} )) && {
      RENK0=''
      RENK1=''
      RENK2=''
      RENK3=''
      RENK4=''
      RENK5=''
      RENK6=''
      RENK7=''
      RENK8=''
    }
}

# Uçbirim pencere başlığını belirle.
ucbirim_basligi() {
  [[ ! -t 1 ]] || [[ ! $DISPLAY ]] && return 0

   case $TERM in
     rxvt*|xterm*|aterm)
       echo -n -e "\033]0;${AD^}: $@\007" ;;
   esac
}

# http://git.archlinux.fr/yaourt.git/tree/src/lib/util.sh.in ( load_lib() )
bilesen_yukle() {
  while [[ $1 ]]
  do
    (( YUKLENEN_BILESENLER[$1] )) && return 0

    if [[ ! -r "${BILESEN_DIZINI}/$1.bash" ]]
    then
        printf "$1.bash dosyası bulunamadı.\n"
        exit 1
    fi

    . "${BILESEN_DIZINI}/$1.bash" || printf "$1.bash bileşeninde sorun var.\n"
    YUKLENEN_BILESENLER[$1]=1
    shift
  done
}

hatali_kullanim() {
  printf "%s: -- \`%s' geçerli değil.\nYardım için '%s --yardım' kullanın.\n" \
    "${AD}" "$1" "${AD}" >&2
  exit 1
}

gorsel_guncelleme_yap() {

  if  [[ -x $(type -p yad) ]]
  then
      (
        guncelleme_yap "$1"
      ) | yad  --progress --undecorated --progress-text="${EZANVERI_ADI} güncelleniyor" \
               --pulsate --auto-close --width=300 --center --fixed \
               --no-buttons --skip-taskbar
  else
      guncelleme_yap "$1"
  fi
}

# Ezanveri dosyası gerektiren işlemlerde ilk önce bu fonksiyon çağrılacak.
#
# ezanveri dosyasının kullanıldığı işlemler için olumsuz durumları denetle.
ezanveri_denetle() {
  local ksatir

  [[ ! -f ${EZANVERI} ]] && { # ezanveri dosyası yoksa
    if (( GUNCELLEME_YAP ))
    then
        bilesen_yukle guncelleyici
        guncelleme_yap
    else
        printf '%s: %s dosyası bulunamadı.\n' \
          "${AD}" "${EZANVERI}"
        exit 1
    fi
  }

  # Bugüne ait tarih ezanveri dosyasında yoksa
  if ! grep -qo "^$(date +%d.%m.%Y)" "${EZANVERI}"
  then
      if (( GUNCELLEME_YAP ))
      then
          bilesen_yukle guncelleyici
          guncelleme_yap
      else
          printf '%s: %s dosyası güncel değil.\n' \
            "${AD}" "${EZANVERI}"
          exit 1
      fi
  fi

  # ezanveri dosyasından tarih bloğunu ayıklayıp son tarih satır no dan bugünkünü çıkardık.
  ksatir=$(gawk '$1 ~ /^[0-9]{2}.[0-9]{2}.[0-9]{4}/ {t++} END{tsatir=t;}; \
    $1 ~ /^[0-9]{2}.[0-9]{2}.[0-9]{4}/ {b++} \
    {if($1 ~ strftime("%d.%m.%Y")) bsatir=b;} END{print(tsatir-bsatir+1)}' "${EZANVERI}")

  (( ksatir <= 7 )) && { # sonuç 7 ya da 7'den küçükse
    (( GUNCELLEME_YAP )) && { bilesen_yukle guncelleyici; guncelleme_yap; }

    # Betiğin mevcut oturum boyunca sadece ilk çalışmada bildirim vermesi
    # için çerez dosya denetimi ekledik. Gün değişimi durumu için (mevcut oturum devam ediyorsa)
    # ayrıca dosyaya tarih uzantısı da ekledik.
    [[ ! -f /tmp/.${AD}_eznvrgncldntle_$(date +%d%m%y) ]] && {
      notify-send "${AD^}" \
        "${EZANVERI_ADI} dosyanız\n\t$ksatir gün\nsonra güncelliğini yitirecek." \
        -i ${AD} -t $GUNCELLEME_BILDIRIM_SURESI"000"
      :> /tmp/.${AD}_eznvrgncldntle_$(date +%d%m%y)
    }
  }
}

# Bugünün ezan vakitlerini ayıkla ve değerleri vakit değişkenlerine ata.
bugun() {
  TARIH=$(date +%d.%m.%Y)
  SAAT=$(date +%H:%M)

  # 08, 09 için oluşan sekizli sayı hatasının önüne geçmek için saatin,
  # sayısal karşılaştırmalarda kullanılması için saati UNIX ikilik saatine çevirdik.
  UNIXSAAT=$(date -d "$SAAT" +%s)

  export $(gawk 'BEGIN{tarih = strftime("%d.%m.%Y")} {if($1 ~ tarih) \
    {printf \
    "sabah_n=%s\ngunes_n=%s\nogle_n=%s\nikindi_n=%s\naksam_n=%s\nyatsi_n=%s"\
    , $2,$3,$4,$5,$6,$7}}' "${EZANVERI}")

  sabah=$(date -d "$sabah_n" +%s)
  gunes=$(date -d "$gunes_n" +%s)
  ogle=$(date -d "$ogle_n" +%s)
  ikindi=$(date -d "$ikindi_n" +%s)
  aksam=$(date -d "$aksam_n" +%s)
  yatsi=$(date -d "$yatsi_n" +%s)
  yeni_gun=$(date -d 'tomorrow 00:00' +%s)
}

# Eğer hedef süre bugün içerisindeyse bu fonksiyon kullanılacak.
bekleme_suresi() {
  bekle=$(( $(date -d "$1" +%s) - $(date +%s) + EZAN_OKUNMA_SURESI_FARKI ))
}

# Eğer hedef süre yarına aitse bu fonksiyon kullanılacak.
bekleme_suresi_yarin() {
  bekle=$(( 86400 - $(date +%s) + $(date -d "$1" +%s) + EZAN_OKUNMA_SURESI_FARKI ))
}

# bekleme süresi fonksiyonlarından gelen bekle değerini
# saat, dakika ve saniyeye çeviren fonksiyon
kalan() {
  kalan_sure=$(printf '%02d saat : %02d dakika : %02d saniye' \
                      $(( bekle / 3600 )) \
                      $(( bekle % 3600 / 60 )) \
                      $(( bekle % 60 )) )
}

indirici() {
  if [[ -x $(type -p wget) ]]
  then
      wget --quiet --tries=3 "$1" -O -
  elif [[ -x $(type -p curl) ]]
  then
      curl -s "$1"
  else
      printf '%s: Bu özellik wget ya da curl gerektirmektedir.\n' "${AD}" >&2
      exit 1
  fi
}
# vim: set ft=sh ts=2 sw=2 et:
